// *******************************************************
// *                                                     *
// *    JAVA  APPLIKATION:  BoxBatterie                  *
// *                                                     *
// *    Aufladen einer virtuellen Batterie durch         *
// *    Schlaege auf den interaktiven KHM Boxsack        *
// *                                                     *
// *         LabPro Sensor: globale Beschleunigung       *
// *         LabPro class:  Boxsack                      *
// *                                                     *
// *******************************************************
// *                                                     *
// *     CodeKit:    author  Jochen Viehoff              *
// *                                                     *
// *******************************************************

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class BoxBatterie {
    
    //  ******************************************
    //     main method
    //  ******************************************
    
    public static void main(String[] args) {
        
        JFrame myFrame = new BoxBatterieFrame();
        myFrame.show();
    }
}

class BoxBatterieFrame extends JFrame implements Runnable,ActionListener {
    
    double accX,accY,accZ,accSum;        // sensor data

    double charge     = 6.0;             // batterie charge state     
    double chargeSpeed= 1.0;             // charge speed  
    
    int   threshold   = 500;            // boxing threshold for recharge
    int   delayAnim   = 10;              // animation speed
    int   dTime       = 10000;            // discharge time in ms, e.g.: 5000=5s*6=30s 
    int   chargeLevel;                   // charge level
        
    static int width  =40;               // display size
    static int height =120;
    
    Boxsack mySack;                      // LabPro sensor
    
    Thread animThread;                   // Thread for animation
    
    String [] imageName = new String[7];  
    Image  [] image     = new Image [7];
    
    Robot mouseRobot;                    // Robot Objekt for remote control
    
    //  ******************************************
    //       constructor
    //  ******************************************
    
    public BoxBatterieFrame() {
        
        setTitle("BoxingBatterie");
        setSize(width,height);
        setBackground(Color.white);
       
        // window closing event
        addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent we){
                System.out.println("WindowClosing");
                System.out.println("Close serial port");
                mySack.serialPort.close();
                System.exit(0);
            }
        }
        );

        // get batterie images
        for (int i=0;i<7;i++){
            imageName[i]="C:\\User\\viehoff\\InterBoxing\\load"+i+".gif";
            image[i] = Toolkit.getDefaultToolkit().getImage(imageName[i]);
        }
        MediaTracker imageTracker = new MediaTracker(new JPanel());
        for (int i=0;i<7;i++)
            imageTracker.addImage(image[i], 0);
        
        try{
            imageTracker.waitForID(0);
        }catch(InterruptedException e){System.out.println(e);}
        
        // start labpro sampling
        Boxsack mySack = new Boxsack();
        
        // start mouse robot
        try {mouseRobot = new Robot();}
        catch(AWTException e) {System.out.println(e);}
        
        // start animation thread
        animThread = new Thread(this);
        animThread.start();                // start animation thread
    }
    
    public void actionPerformed(ActionEvent event) {
        Object source = event.getSource();
    }
    
    //  ******************************************
    //       overwrite paint method
    //  ******************************************
    
    public void paint(Graphics g) {

       if (charge>6) charge=6;
       if (charge<0) charge=0;
        
        g.drawImage(image[(int)charge],30,30,this);
    }
    
    //  ******************************************
    //       overwrite update method
    //  ******************************************
    
    public void update(Graphics g) {
        paint(g);
    }
    
    
    //  ******************************************
    //       animation thread: run methode
    //  ******************************************
    
    public void run() {
        
        Date startTime= new Date();              // get start time
        
        while (animThread != null ) {

            try {
                Thread.sleep(delayAnim);         // sleep a while
            } catch (InterruptedException e) {}

            accX = mySack.getAccelerationX();    // get the acceleration data 
            accY = mySack.getAccelerationY();
            accZ = mySack.getAccelerationZ();

            if (charge==0)
                mouseRobot.mouseMove(0,0);       // empty battery: stop mouse cursor
            
            accSum = accX*accX+accY*accY+accZ*accZ;
            
            if (accSum > threshold){
                charge = charge+chargeSpeed;        // recharge the batterie
            try {
                Thread.sleep(200);         // sleep a while
            } catch (InterruptedException e) {}
            }
            
            Date currentTime = new Date();          // get current time

            // discharge batterie
            if ((currentTime.getTime()-startTime.getTime())>dTime){    
                charge = charge-1.0;
                startTime = currentTime;
            }

            repaint();
        }
    }
}

//  ******************************************
//  ******************************************
