// *******************************************************
// *                                                     *
// *    JAVA  APPLET:   ImagePlayer                      *
// *                                                     *
// *    Umwandeln der Grauwerte in Midi-Noten            *
// *                                                     *
// *******************************************************
// *                                                     *
// *     CodeKit:    author  Jochen Viehoff              *
// *                                                     *
// *******************************************************


import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.net.*;
import java.io.*;


public class ImagePlayer extends Applet implements Runnable,MouseListener,MouseMotionListener {
    
    Image img;
    
    int width  = 400;                            // Breite des Bildes
    int height = 400;                            // Hoehe
    int dim    = width*height;                   // Groesse des Bildes
    
    int[] pixel  = new int [dim];
    
    int cursorX,cursorY;                         // aktuelle Scan-Position
    
    int offset     = 10;                         // tiefster Notenwert
    int spread     = 3;                          // Notenspanne
    
    int tonhoehe;
    int tonlaenge=30;
    int volume=125;
    int instrument=0;
    
    int delayAnim        = 10;                   // animation speed
    Thread animThread;                           // animation thread
    
    MidiInstrument piano;
    
    String imgName = "bild.jpg";           // Name des Bildes
    
    // ****************************************************
    //    Applet: Init Methode
    // ****************************************************
    
    public void init() {
        
        addMouseListener(this);
        addMouseMotionListener(this);
        
        MidiInstrument piano = new MidiInstrument(instrument);
        
        img = getImage(getDocumentBase(), imgName);
        grabPixel();
        
    }
    
    
    // ****************************************************
    //    Applet: Start Methode
    // ****************************************************
    
    public void start() {
        
        cursorX = 0;                                     // init cursor
        cursorY = 200;
        
        if (animThread == null) {                        // start animation thread
            animThread = new Thread(this);
            animThread.start();
        }
    }
    
    // ****************************************************
    //    Applet: Stop Methode
    // ****************************************************
    
    public void stop()  {
        
        if (animThread != null && animThread.isAlive()) {
            animThread.stop();
            animThread = null;                            // stop animation thread
        }
    }
    
    // ****************************************************
    //    Thread: Run Methode
    // ****************************************************
    
    public void run() {
        
        while(animThread != null && animThread.isAlive()) {
            
            try {                                         // wait a second
                animThread.sleep(delayAnim);
            }
            catch(InterruptedException e) {}
            
            cursorX++;                                    // move cursor
            repaint();                                    // redraw image
            
            tonhoehe = (int)(offset+pixel[cursorX+cursorY*width]/spread);     // Tonhoehe aus Bild errechnen
            
            piano.play(tonhoehe,volume,tonlaenge);                   // play tone
        }
        
        animThread = null;
        
    }
    
    // ****************************************************
    //   Ueberschreiben der update-Methode
    // ****************************************************
    
    public void update(Graphics g)
    { paint(g);
    }
    
    // ****************************************************
    //   Ueberschreiben der paint-Methode
    // ****************************************************
    
    public void paint(Graphics g){
        
        g.drawImage(img, 0, 0, this);
        g.drawRect(cursorX,cursorY,5,5);
    }
    
    // ****************************************************
    //   Pixel Grabbing
    // ****************************************************
    
    public void grabPixel() {
        
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(img, 0);
        try { tracker.waitForID(0); }
        catch (InterruptedException e) {}
        
        PixelGrabber grabber=new PixelGrabber(img,0,0,-1,-1,true);
        
        try {
            if (grabber.grabPixels()) {
                int pixels[] = (int[])grabber.getPixels();   // Pixel Grabbing
                for (int x=0;x<dim;x++)
                    pixel[x] = (int)(pixels[x]&0xFF);
            }
        }
        catch (InterruptedException e) {                     // Achtung: IO Fehler
            System.out.print("Error:" + e);
            System.exit(1);
        }
    }
    // ****************************************************
    //   Mouse Events
    // ****************************************************
    
    public void mousePressed(MouseEvent e)   {}
    public void mouseReleased(MouseEvent e)  {}
    
    public void mouseClicked(MouseEvent e)   {
        
        cursorX = e.getX();
        cursorY = e.getY();
    }
    
    public void mouseEntered(MouseEvent e)  {}
    public void mouseExited(MouseEvent e)   {}
    public void mouseDragged(MouseEvent e)  {}
    public void mouseMoved(MouseEvent e)    {}
    
}
