//    ******************************************
//    *                                        *
//    *  package:  LabPro   v0.1               *
//    *  author:   Jochen Viehoff              *
//    *                                        *
//    *  JAVA source code for Vernier LabPro   *
//    *  data collection, single sensor mode   *
//    *                                        *
//    *  communicationAPI required             *
//    *  real-time data collection             *
//    *  binary data (serial port)             *
//    *                                        *
//    *  classes:  Acceleration3D (3 axis)     *
//    *            Temperature                 *
//    *            Mircophone                  *
//    *            MagneticField               *
//    *            MotionDetector              *
//    *            HeartRate                   *
//    *            Barometer                   *
//    *            Humidity                    *
//    *            Current                     *
//    *            Voltage                     *
//    *            VoltageProbe                *
//    *            GasSensor                   *
//    *                                        *
//    ******************************************

// -----------------------------------------------

//    ******************************************
//    * Sensor:   KHM KippKicker               *
//    *                                        *
//    * class:    KippKick                     *
//    *                                        *
//    * methods:  (int)    getLevel()          *
//    *           (void)   moveUp()            *
//    *           (void)   moveDown()          *
//    *           (void)   release()           *
//    *                                        *
//    * parameter: sampleRate                  *
//    *            threadDelay                 *
//    *                                        *
//    ******************************************
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.util.*;
import javax.comm.*;                          // commAPI for serial communication

public class KippKick extends JFrame implements Runnable, SerialPortEventListener,KeyListener {
    
    static Enumeration        portList;       // list for connected ports
    static CommPortIdentifier portId;
    
    static float sampleRate  = 0.05f;         // sample rate for real-time data collection
    
    final int     threadDelay = 1000;          // thread delay
    final int     moveDelay   = 2000;        // delay for moving up and down
    static int    rawDist;                    // raw binary data
    static double dist;                       // distance
    static int    level = 0;                  // level of the table
    
    static double k0 = -51.751;               // gauge parameters
    static double k1 =  0.00174896;
    
    static String message0;                   // commands for LabPro setup
    static String message4;
    static String messageStart;
    static String messageReset;
    static String messageUp,messageDown,messageNull;
    
    static SerialPort   serialPort;
    static OutputStream outputStream;
    static InputStream  inputStream;
    
    Thread readData;
    static boolean data,up,down,release,control;
    
    String []  imageName    = new String[7];
    Image  []  image        = new Image [7];
    
    //    ******************************************
    //      constructor
    //    ******************************************
    
    public KippKick() {
        // set frame
        setTitle("KippKick");
        setSize(600,568);
        setBackground(Color.white);
        show();
        
        // add KeyListener
        addKeyListener(this);
        
        // load the images
        imageName[0]="C:\\User\\viehoff\\KippKick\\kicker-3.jpg";
        image[0] = Toolkit.getDefaultToolkit().getImage(imageName[0]);
        imageName[1]="C:\\User\\viehoff\\KippKick\\kicker-2.jpg";
        image[1] = Toolkit.getDefaultToolkit().getImage(imageName[1]);
        imageName[2]="C:\\User\\viehoff\\KippKick\\kicker-1.jpg";
        image[2] = Toolkit.getDefaultToolkit().getImage(imageName[2]);
        imageName[3]="C:\\User\\viehoff\\KippKick\\kicker0.jpg";
        image[3] = Toolkit.getDefaultToolkit().getImage(imageName[3]);
        imageName[4]="C:\\User\\viehoff\\KippKick\\kicker1.jpg";
        image[4] = Toolkit.getDefaultToolkit().getImage(imageName[4]);
        imageName[5]="C:\\User\\viehoff\\KippKick\\kicker2.jpg";
        image[5] = Toolkit.getDefaultToolkit().getImage(imageName[5]);
        imageName[6]="C:\\User\\viehoff\\KippKick\\kicker3.jpg";
        image[6] = Toolkit.getDefaultToolkit().getImage(imageName[6]);
        
        
        MediaTracker imageTracker = new MediaTracker(new JPanel());
        for (int i=0;i<6;i++)
            imageTracker.addImage(image[i], 0);
        
        // wait for the images
        try{
            imageTracker.waitForID(0);
        }catch(InterruptedException e){System.out.println(e);}
        
        portList = CommPortIdentifier.getPortIdentifiers();
        
        // locking for serial ports
        
        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                System.out.println("Found: SerialPort");
                if (portId.getName().equals("COM1")) {
                    System.out.println("Found COM1");
                    
                    // open port COM1
                    try {
                        serialPort = (SerialPort) portId.open("LabPro", 2000);
                        System.out.println("open port: done");
                    } catch (PortInUseException e) {System.out.println(e);}
                    
                    // get the streams from/to port COM1
                    try {
                        inputStream = serialPort.getInputStream();
                        outputStream= serialPort.getOutputStream();
                    } catch (IOException e) {System.out.println(e);}
                    
                    try {
                        serialPort.addEventListener(this);
                    } catch (TooManyListenersException e) {System.out.println(e);}
                    
                    // start listening to the port
                    serialPort.notifyOnDataAvailable(true);
                    
                    // set port parameters
                    try {
                        serialPort.setSerialPortParams(38400,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
                    } catch (UnsupportedCommOperationException e) {System.out.println(e);}
                    
                    // start thread for checking data rate
                    readData = new Thread(this);
                    readData.start();
                }
            }
        }
    }
    //  ******************************************
    //       paint method
    //  ******************************************
    
    public void paint(Graphics g) {
        
        g.drawImage(image[level+3],0,100,this);
    }
    
    //  ******************************************
    //       update method
    //  ******************************************
    
    public void update(Graphics g) {
        paint(g);
    }
    
    //    ******************************************
    //      public methods
    //    ******************************************
    
    public static int getLevel() {
        
        // send back level of table
        return level;
    }
    
    public static void moveUp() {
        up      = true;
        down    = false;
        release = false;
        return;
    }
    public static void moveDown() {
        down    = true;
        up      = false;
        release = false;
        return;
    }
    public static void release() {
        release = true;
        down    = false;
        up      = false;
        return;
    }
    
    public void run() {
        
        control=true;
        // reset LabPro
        messageReset = "s{0}\n";
        System.out.println("Reset LabPro");
        
        // build commands for channel set up
        message0 = "s{1,1,1,0,0,1}\n";
        message4 = "s{4,0,-1}\n";            //request binary data !!!!
        
        messageUp    = "s{2001,1}\n";        // set labpro digital output
        messageDown  = "s{2001,2}\n";        // 1 = 0001   2 = 0010
        messageNull  = "s{2001,0}\n";        // 0 = 0000
        
        // start LabPro: real time sampling
        messageStart = "s{3,"+sampleRate+",-1}\n";
        System.out.println("start real time sampling: t="+sampleRate);
        
        // send commands to LabPro
        //        try {
        //            outputStream.write(messageReset.getBytes());
        //            outputStream.write(message0.getBytes());
        //            outputStream.write(message4.getBytes());
        //            outputStream.write(messageStart.getBytes());
        //        } catch (IOException e) {}
        
        while (readData != null) {
            
            if (control&&up&&(level!=3)) {
                try {outputStream.write(messageUp.getBytes()); }
                catch (IOException e) {}
                try {Thread.sleep(moveDelay);}
                catch (InterruptedException e) {}
                up   =false;
                level++;
                try {outputStream.write(messageNull.getBytes());}
                catch (IOException e) {}
            }
            if (control&&down&&(level!=-3)) {
                try {outputStream.write(messageDown.getBytes());}
                catch (IOException e) {}
                try {Thread.sleep(moveDelay);}
                catch (InterruptedException e) {}
                down   =false;
                level--;
                try {outputStream.write(messageNull.getBytes());}
                catch (IOException e) {}
            }
            if (control&&release&&(level>0)) {
                try {outputStream.write(messageDown.getBytes());}
                catch (IOException e) {}
                try {Thread.sleep(moveDelay);}
                catch (InterruptedException e) {}
                release=false;
                level--;
                try {outputStream.write(messageNull.getBytes());}
                catch (IOException e) {}
            }
            if (control&&release&&(level<0)) {
                try {outputStream.write(messageUp.getBytes());}
                catch (IOException e) {}
                try {Thread.sleep(moveDelay);}
                catch (InterruptedException e) {}
                release=false;
                level++;
                try {outputStream.write(messageNull.getBytes());}
                catch (IOException e) {}
            }
            System.out.println(level);
            repaint();
            
            try {Thread.sleep(moveDelay);}
            catch (InterruptedException e) {}
            
        }
        
        //        close port if thread has died
        serialPort.close();
    }
    
    //    ******************************************
    //      SerialEvent handling
    //    ******************************************
    
    public void serialEvent(SerialPortEvent e) {
        switch (e.getEventType()) {
            
            case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
                break;
            case SerialPortEvent.DATA_AVAILABLE:
                
                final int BUFFSIZE=4;
                int lenserial=0;
                byte bufserial [] = new byte[BUFFSIZE];
                
                // Read binary data until -1 is returned
                try {
                    while (inputStream.available() > 0) {
                        lenserial = inputStream.read(bufserial);
                        if (lenserial==4){
                            rawDist = ((0xff & bufserial[0])<<8) | ((0xff & bufserial[1]));
                        }
                    }
                }
                catch (IOException ex) {
                    System.err.println(ex);
                    return;
                }
                break;
                
                // If break event append BREAK RECEIVED message.
                
            case SerialPortEvent.BI:
                System.out.println("\n--- BREAK RECEIVED ---\n");
        }
    }
    
    // ****************************************************
    //   KeyEvents
    // ****************************************************
    
    public void keyPressed(KeyEvent evt)  {
        
        int keyCode;
        keyCode = evt.getKeyCode();
        
        // hand control UP / DOWN
        if (keyCode==KeyEvent.VK_UP) {
            try {outputStream.write(messageUp.getBytes()); }
            catch (IOException e) {}
            System.out.println("Move UP");
        }
        if (keyCode==KeyEvent.VK_DOWN) {
            try {outputStream.write(messageDown.getBytes()); }
            catch (IOException e) {}
            System.out.println("Move DOWN");
        }
        if (keyCode==KeyEvent.VK_SPACE) {
            try {outputStream.write(messageNull.getBytes()); }
            catch (IOException e) {}
            control=!control;
            System.out.println("STOP");
        }
    }
    public void keyReleased(KeyEvent evt) {
    }
    public void keyTyped(KeyEvent evt)    {}
    
    
    
}
