// *******************************************************
// *                                                     *
// *    JAVA  APPLET:   RotateImage                      *
// *                                                     *
// *    Bildrotation (pixel fuer pixel, Mouseinteraktion)*
// *                                                     *
// *******************************************************
// *                                                     *
// *     CodeKit:    author  Jochen Viehoff              *
// *                                                     *
// *******************************************************


import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.net.*;
import java.io.*;


public class RotateImage extends Applet implements Runnable,MouseListener,MouseMotionListener{
    
    Image image;
    Image buffer;
    
    int width  = 400;                            // Breite des Bildes
    int height = 400;                            // Hoehe
    int dim    = width*height;                   // Groesse des Bildes
    int red,green,blue;                          // Farbkanaele
    
    int centerX = 200;                           // Mittelpunkt der Rotation
    int centerY = 200;
    
    int newX,newY;
    int phi=0;                                   // Rotationswinkel in Grad
    
    double sinPhi[] = new double[360];
    double cosPhi[] = new double[360];	         // Umrechnungskonstanten
    
    int[] pixel  = new int [dim];                // Pixel Array
    int[] temp   = new int [dim];                // Zwischenspeicher
    
    private Thread rotateThread;                // Thread fuer die Animation
    
    
    // ****************************************************
    //    Applet: Init Methode
    // ****************************************************
    
    public void init() {
        
        addMouseListener(this);
        addMouseMotionListener(this);
        
        image = getImage(getDocumentBase(), "testrotation.jpg");
        
        for (int i=0;i<360;i++) {
            sinPhi[i] = Math.sin(i*3.1415927/180);      // precalculate sin/cos values
            cosPhi[i] = Math.cos(i*3.1415927/180);
        }
    }
    
    
    // ****************************************************
    //    Applet: Start Methode
    // ****************************************************
    
    public void start() {
        
        grabPixel();
        
        for (int i=0;i<dim;i++)
            temp[i]=pixel[i];
        
        repaint();
        
        if (rotateThread == null) {
            rotateThread = new Thread(this);
            rotateThread.start();
        }
        
    }
    
    // ****************************************************
    //    Applet: Stop Methode
    // ****************************************************
    
    public void stop()  {
        
        if (rotateThread != null && rotateThread.isAlive()) {
            rotateThread.stop();
            rotateThread = null;
        }
    }
    
    // ****************************************************
    //    Thread: Run Methode
    // ****************************************************
    
    public void run() {
        
        while(rotateThread != null) {
            
            try {
                rotateThread.sleep(10);
            }
            catch(InterruptedException e) {}
            
            for (int x=40;x<width-40;x++) {
                for (int y=40;y<height-40;y++) {
                    
                    newX=(int)(((x-centerX)*cosPhi[phi]-(y-centerY)*sinPhi[phi])+centerX);
                    newY=(int)(((x-centerX)*sinPhi[phi]+(y-centerY)*cosPhi[phi])+centerY);
                    
                    if ( (newY*width+newX)<dim && (newY*width+newX)>=0)
                        temp[y*width+x]=pixel[newY*width+newX];
                }
            }
            
            repaint();
            
            
        }
        
        rotateThread = null;
    }
    
    // ****************************************************
    //   Ueberschreiben der update-Methode
    // ****************************************************
    
    public void update(Graphics g)
    { paint(g);
    }
    
    // ****************************************************
    //   Ueberschreiben der paint-Methode
    // ****************************************************
    
    public void paint(Graphics g){
        
        
        buffer = createImage(new MemoryImageSource(width,height,temp,0,width));
        g.drawImage(buffer,0,0,null);
    }
    
    
    // ****************************************************
    //   Pixel Grabbing
    // ****************************************************
    
    public void grabPixel() {
        
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(image, 0);
        try { tracker.waitForID(0); }
        catch (InterruptedException e) {}
        
        PixelGrabber grabber=new PixelGrabber(image,0,0,-1,-1,true);
        
        try {
            if (grabber.grabPixels()) {
                int grabPixel[] = (int[])grabber.getPixels();   // Pixel Grabbing
                
                for (int x=0;x<dim;x++)
                    pixel[x] = grabPixel[x];
            }
        }
        catch (InterruptedException e) {                     // Achtung: IO Fehler
            System.out.print("Error:" + e);
            System.exit(1);
        }
    }
    
    // ****************************************************
    //   Mouse Events
    // ****************************************************
    
    public void mousePressed(MouseEvent e)  {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseClicked(MouseEvent e)  {}
    public void mouseEntered(MouseEvent e)  {}
    public void mouseExited(MouseEvent e)   {}
    public void mouseDragged(MouseEvent e)  {}
    public void mouseMoved(MouseEvent e)    {
        phi = e.getX();
        if (phi>360) phi=359;
    }
    
}
