// *******************************************************
// *                                                     *
// *    JAVA  APPLIKATION:   Runde10                     *
// *                                                     *
// *    ein kleines Boxspiel fuer den                    *
// *    interaktiven KHM Boxsack                         *
// *    integrale Gesamtbeschleunigung pro Minute        *
// *    entspricht der Leistung des Spielers             *
// *                                                     *
// *******************************************************
// *                                                     *
// *     CodeKit:    author  Jochen Viehoff              *
// *                                                     *
// *******************************************************

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.AudioClip;
import java.net.URL;
import java.net.MalformedURLException;


public class Runde10 {
    
    //  ******************************************
    //     main method
    //  ******************************************
    
    public static void main(String[] args) {
        
        JFrame myFrame = new Runde10Frame();
        myFrame.show();
    }
}

class Runde10Frame extends JFrame implements Runnable,ActionListener {
    
    static int width  =1040;            // display size
    static int height =800;
    
    Boxsack mySack;                     // LabPro sensor
    double  accSum;                     // sensor data
    double  threshold   =0.22;            // boxing threshold
    
    Thread animThread;                  // Thread for animation
    boolean clearScreen;                // control flag
    
    int        totalImages  = 31;
    String []  imageName    = new String[totalImages];
    Image  []  image        = new Image [totalImages];
    
    int        totalObjects = 5;
    boolean [] show         = new boolean[totalObjects];
    int     [] numImage     = new int    [totalObjects];
    int     [] posX         = new int    [totalObjects];
    int     [] posY         = new int    [totalObjects];
    
    Color bgColor = new Color(100,100,100);
    
    SoundList soundList;
    String titleFile    = "rocky3.wav";
    String punchFile    = "punch.wav";
    String gongFile     = "gong.wav";
    String applauseFile = "applaus.wav";
    String trommelFile  = "trommel.au";
    String cheerFile    = "cheer.wav";
    String champFile    = "glory.au";
    
    AudioClip title,punch,gong,applaus,trommel,cheer,champ;
    URL       codeBase;
    
    //  ******************************************
    //       constructor
    //  ******************************************
    
    public Runde10Frame() {
        
        // set frame
        setTitle("Runde 10");
        setSize(width,height);
        setBackground(bgColor);
        
        // load the sound clips
        startLoadingSounds();
        
        // window closing event
        addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent we){
                System.out.println("WindowClosing");
                System.out.println("Close serial port");
                mySack.serialPort.close();
                System.exit(0);
            }
        }
        );
        
        // load the images
        for (int i=0;i<totalImages-1;i++){
            imageName[i]="C:\\User\\viehoff\\InterBoxing\\BoxBilder\\pic"+i+".jpg";
            image[i] = Toolkit.getDefaultToolkit().getImage(imageName[i]);
        }
        
        imageName[9]="C:\\User\\viehoff\\InterBoxing\\BoxBilder\\pic.gif";
        image[9] = Toolkit.getDefaultToolkit().getImage(imageName[9]);
        
        MediaTracker imageTracker = new MediaTracker(new JPanel());
        for (int i=0;i<totalImages-1;i++)
            imageTracker.addImage(image[i], 0);
        
        // wait for the images
        try{
            imageTracker.waitForID(0);
        }catch(InterruptedException e){System.out.println(e);}
        
        // start labpro sampling
        Boxsack mySack = new Boxsack();
        
        // start animation thread
        animThread = new Thread(this);
        animThread.start();
    }
    
    public void actionPerformed(ActionEvent event) {
        Object source = event.getSource();
    }
    
    //  ******************************************
    //       paint method
    //  ******************************************
    
    public synchronized void paint(Graphics g) {
        
        g.setColor(bgColor);
        if (show[0]) g.drawImage(image[numImage[0]],posX[0],posY[0],this);
        if (show[1]) g.drawImage(image[numImage[1]],posX[1],posY[1],this);
        if (show[2]) g.drawImage(image[numImage[2]],posX[2],posY[2],this);
        if (show[3]) g.drawImage(image[numImage[3]],posX[3],posY[3],this);
        if (show[4]) g.drawImage(image[numImage[4]],posX[4],posY[4],this);
        if (clearScreen) g.fillRect(0,0,width,height);
        clearScreen=false;
    }
    
    //  ******************************************
    //       update method
    //  ******************************************
    
    public synchronized void update(Graphics g) {
        paint(g);
    }
    
    //  ******************************************
    //       animation thread: run methode
    //  ******************************************
    
    public void run() {
        
        boolean scene;
        int     count;
        double  score1=0.0,score2=0.0,scoreLow=0.0;
        
        // get sound clips from soundlist
        title   = soundList.getClip(titleFile);
        punch   = soundList.getClip(punchFile);
        gong    = soundList.getClip(gongFile);
        applaus = soundList.getClip(applauseFile);
        trommel = soundList.getClip(trommelFile);
        cheer   = soundList.getClip(cheerFile);
        champ   = soundList.getClip(champFile);
        
        while (animThread != null ) {
            
            //  ******************************************
            //    play Intro
            //  ******************************************
            
            title.loop();
            clearFrame();
            wait(10000);
            
            show[0] = true ; numImage[0] = 0 ; posX[0] = 50 ; posY[0] = 50;
            repaint();
            wait(9000);
            //            waitForHit();
            
            clearFrame();
            
            show[0] = true ; numImage[0] = 26 ; posX[0] = 295 ; posY[0]=80;
            repaint();
            wait(8000);
            
            clearFrame();
            
            show[0] = true ; numImage[0] = 9 ; posX[0] = 412 ; posY[0]=334;
            repaint();
            
            waitForHit();
            title.stop();
            wait(3000);
            
            //  ******************************************
            //    Competition: player 1
            //  ******************************************
            
            clearFrame();
            
            show[0] = true ; numImage[0] = 13; posX[0] = 50  ; posY[0] = 80;
            show[1] = true ; numImage[1] = 1 ; posX[1] = 300 ; posY[1] = 80;
            show[2] = true ; numImage[2] = 9 ; posX[2] = 50  ; posY[2] = 364;
            show[3] = true ; numImage[3] = 3 ; posX[3] = 50  ; posY[3] = 474;
            repaint();
            wait(2000);
            
            waitForHit();
            title.stop();
            gong.play();
            
            show[2] = true ; numImage[2] = 11 ; posX[2] = 50 ; posY[2] = 364;
            repaint();
            
            scene      = true;
            score1     = 0.0;
            count      = 0;
            Date start = new Date();
            
            while (scene) {
                accSum = mySack.getAccSquare();
                if ((accSum>threshold)&&(accSum<1.0)) {
                    score1=score1+accSum;
                    punch.play();
                    numImage[1] = 2;
                    repaint();
                    wait(200);
                    numImage[1] = 1;
                    repaint();
                }
                wait(1);
                Date time = new Date();
                if ((time.getTime()-start.getTime())>4000){
                    start = time;
                    count++;
                    numImage[0]=13+count;
                    repaint();
                }
                if (count==12)
                    scene = false;
            }
            
            numImage[2] = 12 ; posX[2] = 50 ; posY[2] = 364;
            repaint();
            gong.play();
            wait(1000);
            applaus.play();
            wait(5000);
            
            //  ******************************************
            //    Competition: player2
            //  ******************************************
            
            clearFrame();
            
            show[0] = true ; numImage[0] = 13; posX[0] = 50  ; posY[0] = 80;
            show[1] = true ; numImage[1] = 1 ; posX[1] = 300 ; posY[1] = 80;
            show[2] = true ; numImage[2] = 9 ; posX[2] = 50  ; posY[2] = 364;
            show[3] = true ; numImage[3] = 4 ; posX[3] = 50  ; posY[3] = 474;
            repaint();
            wait(2000);
            
            waitForHit();
            gong.play();
            show[2] = true ; numImage[2] = 11 ; posX[2] = 50 ; posY[2] = 364;
            repaint();
            
            scene      = true;
            score2     = 0.0;
            count      = 0;
            Date start2 = new Date();
            
            while (scene) {
                accSum = mySack.getAccSquare();
                if ((accSum>threshold)&&(accSum<1.0)) {
                    score2=score2+accSum;
                    punch.play();
                    numImage[1] = 2;
                    repaint();
                    wait(200);
                    numImage[1] = 1;
                    repaint();
                }
                wait(1);
                Date time2 = new Date();
                if ((time2.getTime()-start2.getTime())>4000){
                    start2 = time2;
                    count++;
                    numImage[0]=13+count;
                    repaint();
                }
                if (count==12)
                    scene = false;
            }
            
            numImage[2] = 12 ; posX[2] = 50 ; posY[2] = 364;
            repaint();
            gong.play();
            wait(1000);
            applaus.play();
            wait(5000);
            
            //  ******************************************
            //    Finale - Siegerehrung
            //  ******************************************
            
            clearFrame();
            wait(4000);
            
            show[0] = true ; numImage[0] = 0 ; posX[0] = 50 ; posY[0]  = 50;
            show[1] = true ; numImage[1] = 3 ; posX[1] = 550 ; posY[1] = 50;
            show[2] = true ; numImage[2] = 4 ; posX[2] = 780 ; posY[2] = 50;
            show[3] = true ; numImage[3] = 5 ; posX[3] = 600 ; posY[3] = 768;
            show[4] = true ; numImage[4] = 6 ; posX[4] = 830 ; posY[4] = 768;
            repaint();
            trommel.play();
            wait(2000);
            
            applaus.loop();
            if (score1>score2)
                scoreLow = score2/score1 * 500;
            else
                scoreLow = score1/score2 * 500;
            
            for (int i=height;i>(int)(height-scoreLow);i--){
                posY[3] = i;
                posY[4] = i;
                wait(25);
                repaint();
            }
            applaus.stop();
            
            cheer.play();
            
            if(score1>score2) {
                for (int i=(int)(height-scoreLow);i>300;i--){
                    posY[3] = i;
                    wait(25);
                    repaint();
                }
            }
            else {
                for (int i=(int)(height-scoreLow);i>300;i--){
                    posY[4] = i;
                    wait(25);
                    repaint();
                }
            }
            
            wait(2000);
            clearFrame();
            
            if (score1>=score2){
                show[0] = true ; numImage[0] = 27 ; posX[0] = 80 ; posY[0] = 100;
                show[1] = true ; numImage[1] = 7 ; posX[1] = 520 ; posY[1] = 100;
                repaint();
            }
            else {
                show[0] = true ; numImage[0] = 28 ; posX[0] = 80 ; posY[0] = 100;
                show[1] = true ; numImage[1] = 8 ; posX[1] = 520 ; posY[1] = 100;
                repaint();
            }
            champ.play();
            wait(15000);
            clearFrame();
            
            show[0] = true ; numImage[0] = 29 ; posX[0] = 50 ; posY[0] = 370;
            repaint();
            wait(15000);
            champ.stop();
            
            //  ******************************************
            //    end of competition
            //  ******************************************
        }
    }
    
    //  ******************************************
    //    public method: wait (millisec)
    //  ******************************************
    
    public void wait(int w) {
        
        try {
            Thread.sleep(w);                   // sleep a while
        } catch (InterruptedException e) {}
        
        return;
    }
    //  ******************************************
    //    public method: waitForHit
    //  ******************************************
    
    public void waitForHit() {
        
        boolean waiting = true;
        while (waiting){
            accSum = mySack.getAccSquare();
            wait(50);
            if ((accSum>threshold)&&(accSum<1.0)) waiting=false;
            
        }
        wait(300);
        
        return;
    }
    //  ******************************************
    //    public method: clearFrame
    //  ******************************************
    
    public void clearFrame(){
        
        clearObjects();
        clearScreen=true;
        repaint();
    }
    
    //  ******************************************
    //    public method: clearObjects
    //  ******************************************
    
    public void clearObjects(){
        
        for (int i=0;i<totalObjects;i++)
            show[i] = false;
    }
    
    //  ******************************************
    //    public method: startLoadingSounds
    //  ******************************************
    
    public void startLoadingSounds() {
        //Start asynchronous sound loading.
        try {
            codeBase = new URL("file:C:/User/viehoff/InterBoxing/BoxBilder/");
        } catch (MalformedURLException e) {
            System.err.println(e.getMessage());
        }
        soundList = new SoundList(codeBase);
        soundList.startLoading(titleFile);
        soundList.startLoading(punchFile);
        soundList.startLoading(gongFile);
        soundList.startLoading(applauseFile);
        soundList.startLoading(trommelFile);
        soundList.startLoading(cheerFile);
        soundList.startLoading(champFile);
        
    }
}

//  ******************************************
//  ******************************************
